'use strict';

exports.__esModule = true;
function reduxSlimAsync(_ref) {
  var dispatch = _ref.dispatch,
      getState = _ref.getState;

  return function (next) {
    return function (action) {
      var types = action.types,
          callAPI = action.callAPI,
          _action$formatData = action.formatData,
          formatData = _action$formatData === undefined ? function (res) {
        return res;
      } : _action$formatData,
          _action$shouldCallAPI = action.shouldCallAPI,
          shouldCallAPI = _action$shouldCallAPI === undefined ? function () {
        return true;
      } : _action$shouldCallAPI,
          _action$payload = action.payload,
          payload = _action$payload === undefined ? {} : _action$payload;


      if (!types) return next(action);
      if (!Array.isArray(types) || types.length !== 3 || !types.every(function (type) {
        return typeof type === 'string';
      })) {
        throw new Error('Expected an array of three string types.');
      }
      if (typeof callAPI !== 'function') {
        throw new Error('Expected callAPI to be a function.');
      }
      if (typeof formatData !== 'function') {
        throw new Error('Expected formatData to be a function.');
      }
      if (typeof shouldCallAPI !== 'function') {
        throw new Error('Expected shouldCallAPI to be a function.');
      }
      if (!shouldCallAPI(getState())) return null;

      var pendingType = types[0],
          successType = types[1],
          errorType = types[2];


      dispatch(Object.assign({}, payload, { type: pendingType }));

      return callAPI().then(function (response) {
        var formattedData = formatData(response);
        if (typeof formattedData !== 'object') {
          throw new Error('Expected formatData to return an object.');
        }

        dispatch(Object.assign({}, payload, formattedData, { type: successType }));

        return Promise.resolve(getState());
      })['catch'](function (error) {
        dispatch(Object.assign({}, payload, {
          error: error,
          type: errorType
        }));

        return Promise.reject(getState());
      });
    };
  };
}

exports['default'] = reduxSlimAsync;